/// <reference types="@types/codemirror" />
export import CodeMirror = require('codemirror');
import { CodeMirrorAdapter, ITextEditorOptions } from 'lsp-editor-adapter';
import * as lsProtocol from 'vscode-languageserver-protocol';
import { FreeTooltip } from '../free_tooltip';
import { CompletionTriggerKind } from '../lsp';
import { VirtualEditor } from '../virtual/editor';
import { VirtualDocument } from '../virtual/document';
import { IEditorPosition, IRootPosition } from '../positioning';
import { LSPConnection } from '../connection';
export declare type KeyModifier = 'Alt' | 'Control' | 'Shift' | 'Meta' | 'AltGraph';
interface IEditorRange {
    start: IEditorPosition;
    end: IEditorPosition;
    editor: CodeMirror.Editor;
}
export declare class CodeMirrorAdapterExtension extends CodeMirrorAdapter {
    protected create_tooltip: (markup: lsProtocol.MarkupContent, cm_editor: CodeMirror.Editor, position: IEditorPosition) => FreeTooltip;
    protected invoke_completer: (kind: CompletionTriggerKind) => void;
    private virtual_document;
    connection: LSPConnection;
    editor: VirtualEditor;
    protected highlight_markers: CodeMirror.TextMarker[];
    private marked_diagnostics;
    private _tooltip;
    private show_next_tooltip;
    private last_hover_response;
    private last_hover_character;
    private unique_editor_ids;
    private signature_character;
    private last_change;
    constructor(connection: LSPConnection, options: ITextEditorOptions, editor: VirtualEditor, create_tooltip: (markup: lsProtocol.MarkupContent, cm_editor: CodeMirror.Editor, position: IEditorPosition) => FreeTooltip, invoke_completer: (kind: CompletionTriggerKind) => void, virtual_document: VirtualDocument);
    private _completionCharacters;
    private _signatureCharacters;
    readonly completionCharacters: string[];
    readonly signatureCharacters: string[];
    protected hover_character: IRootPosition;
    handleGoTo(locations: any): void;
    handleCompletion(completions: lsProtocol.CompletionItem[]): void;
    protected static get_markup_for_hover(response: lsProtocol.Hover): lsProtocol.MarkupContent;
    protected remove_range_highlight(): void;
    protected remove_tooltip(): void;
    handleHover(response: lsProtocol.Hover): void;
    get_cm_editor(position: IRootPosition): CodeMirror.Editor;
    get_language_at(position: IEditorPosition, editor: CodeMirror.Editor): any;
    protected get_markup_for_signature_help(response: lsProtocol.SignatureHelp, language: string): lsProtocol.MarkupContent;
    /**
     * A temporary workaround for the LSP servers returning plain text (e.g. docstrings)
     * (providing not-the-best UX) instead of markdown and me being unable to force
     * them to return markdown instead.
     */
    private markdown_from_signature;
    handleSignature(response: lsProtocol.SignatureHelp): void;
    updateAfterChange(): Promise<boolean>;
    invalidateLastChange(): void;
    handleChange(cm: CodeMirror.Editor, change: CodeMirror.EditorChange): void;
    handleHighlight(items: lsProtocol.DocumentHighlight[]): void;
    protected onCursorActivity(): void;
    protected range_to_editor_range(range: lsProtocol.Range, cm_editor?: CodeMirror.Editor): IEditorRange;
    protected editor_range_for_hover(range: lsProtocol.Range): IEditorRange;
    protected highlight_range(range: IEditorRange, class_name: string): CodeMirror.TextMarker;
    protected position_from_mouse(ev: MouseEvent): IRootPosition;
    protected is_token_empty(token: CodeMirror.Token): boolean;
    _handleMouseOver(event: MouseEvent): void;
    handleMouseOver(event: MouseEvent): void;
    protected collapse_overlapping_diagnostics(diagnostics: lsProtocol.Diagnostic[]): Map<lsProtocol.Range, lsProtocol.Diagnostic[]>;
    handleDiagnostic(response: lsProtocol.PublishDiagnosticsParams): void;
    private transform_virtual_position_to_root_position;
}
export {};
