"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const codemirror_1 = require("./codemirror");
const file_editor_1 = require("../virtual/editors/file_editor");
const codemirror_2 = require("@jupyterlab/codemirror");
const codeeditor_1 = require("@jupyterlab/codeeditor");
const connection_1 = require("../connection");
describe('CodeMirrorAdapterExtension', () => {
    const factoryService = new codemirror_2.CodeMirrorEditorFactory();
    let host;
    let model;
    let ce_editor;
    let virtual_editor;
    beforeEach(() => {
        host = document.createElement('div');
        document.body.appendChild(host);
        model = new codeeditor_1.CodeEditor.Model();
        ce_editor = factoryService.newDocumentEditor({ host, model });
        virtual_editor = new file_editor_1.VirtualFileEditor('python', 'x.py', ce_editor.editor);
    });
    afterEach(() => {
        document.body.removeChild(host);
    });
    describe('Works with VirtualFileEditor', () => {
        let adapter;
        beforeEach(() => {
            let connection = new connection_1.LSPConnection({
                languageId: 'python',
                serverUri: '',
                documentUri: '/x.py',
                rootUri: '/',
                documentText: () => {
                    virtual_editor.update_documents();
                    return virtual_editor.virtual_document.value;
                }
            });
            adapter = new codemirror_1.CodeMirrorAdapterExtension(connection, {}, virtual_editor, (markup, cm_editor, position) => {
                return null;
            }, () => {
                return;
            }, virtual_editor.virtual_document);
        });
        it('renders inspections', () => __awaiter(this, void 0, void 0, function* () {
            ce_editor.model.value.text = ' foo \n bar \n baz ';
            yield virtual_editor.update_documents();
            let markers;
            markers = ce_editor.editor.getDoc().getAllMarks();
            chai_1.expect(markers.length).to.equal(0);
            adapter.handleDiagnostic({
                uri: '',
                diagnostics: [
                    {
                        range: {
                            start: { line: 0, character: 1 },
                            end: { line: 0, character: 4 }
                        },
                        message: 'Undefined symbol'
                    }
                ]
            });
            let marks = ce_editor.editor.getDoc().getAllMarks();
            chai_1.expect(marks.length).to.equal(1);
        }));
    });
});
