"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const jupyterlab_1 = require("./jupyterlab");
const completion_1 = require("../completion");
const file_editor_1 = require("../virtual/editors/file_editor");
class FileEditorAdapter extends jupyterlab_1.JupyterLabWidgetAdapter {
    get document_path() {
        return this.widget.context.path;
    }
    get language() {
        return this.jumper.language;
    }
    get ce_editor() {
        return this.editor.editor;
    }
    get cm_editor() {
        return this.ce_editor.editor;
    }
    find_ce_editor(cm_editor) {
        return this.editor.editor;
    }
    constructor(editor_widget, jumper, app, completion_manager, rendermime_registry) {
        super(app, editor_widget, rendermime_registry, 'completer:invoke-file');
        this.jumper = jumper;
        this.editor = editor_widget.content;
        this.virtual_editor = new file_editor_1.VirtualFileEditor(this.language, this.document_path, this.cm_editor);
        this.connect_contentChanged_signal();
        console.log('LSP: file ready for connection:', this.path);
        this.connect_document(this.virtual_editor.virtual_document).then(() => {
            this.current_completion_connector = new completion_1.LSPConnector({
                editor: this.editor.editor,
                connections: this.connections,
                virtual_editor: this.virtual_editor
            });
            completion_manager.register({
                connector: this.current_completion_connector,
                editor: this.editor.editor,
                parent: editor_widget
            });
        });
    }
    get path() {
        return this.widget.context.path;
    }
}
exports.FileEditorAdapter = FileEditorAdapter;
