import { CodeMirror, CodeMirrorAdapterExtension } from './codemirror';
import { JupyterFrontEnd } from '@jupyterlab/application';
import { CodeJumper } from '@krassowski/jupyterlab_go_to_definition/lib/jumpers/jumper';
import { CodeEditor } from '@jupyterlab/codeeditor';
import { IRenderMimeRegistry } from '@jupyterlab/rendermime';
import * as lsProtocol from 'vscode-languageserver-protocol';
import { FreeTooltip } from '../free_tooltip';
import { IDocumentWidget } from '@jupyterlab/docregistry';
import { VirtualEditor } from '../virtual/editor';
import { VirtualDocument } from '../virtual/document';
import { Signal } from '@phosphor/signaling';
import { IEditorPosition, IRootPosition, IVirtualPosition } from '../positioning';
import { LSPConnection } from '../connection';
import { LSPConnector } from '../completion';
import { CompletionTriggerKind } from '../lsp';
interface IDocumentConnectionData {
    document: VirtualDocument;
    connection: LSPConnection;
}
interface IContext {
    document: VirtualDocument;
    connection: LSPConnection;
    virtual_position: IVirtualPosition;
    root_position: IRootPosition;
}
/**
 * Foreign code: low level adapter is not aware of the presence of foreign languages;
 * it operates on the virtual document and must not attempt to infer the language dependencies
 * as this would make the logic of inspections caching impossible to maintain, thus the WidgetAdapter
 * has to handle that, keeping multiple connections and multiple virtual documents.
 */
export declare abstract class JupyterLabWidgetAdapter {
    app: JupyterFrontEnd;
    connections: Map<VirtualDocument.id_path, LSPConnection>;
    documents: Map<VirtualDocument.id_path, VirtualDocument>;
    jumper: CodeJumper;
    protected adapters: Map<VirtualDocument.id_path, CodeMirrorAdapterExtension>;
    protected rendermime_registry: IRenderMimeRegistry;
    widget: IDocumentWidget;
    private readonly invoke_command;
    protected document_connected: Signal<JupyterLabWidgetAdapter, IDocumentConnectionData>;
    protected abstract current_completion_connector: LSPConnector;
    private ignored_languages;
    protected constructor(app: JupyterFrontEnd, widget: IDocumentWidget, rendermime_registry: IRenderMimeRegistry, invoke: string);
    abstract virtual_editor: VirtualEditor;
    abstract readonly document_path: string;
    abstract readonly language: string;
    readonly root_path: string;
    abstract find_ce_editor(cm_editor: CodeMirror.Editor): CodeEditor.IEditor;
    invoke_completer(kind: CompletionTriggerKind): void;
    readonly main_connection: LSPConnection;
    private retry_to_connect;
    protected on_lsp_connected(data: IDocumentConnectionData): Promise<void>;
    protected connect_document(virtual_document: VirtualDocument): Promise<void>;
    document_changed(virtual_document: VirtualDocument): void;
    private connect_adapter;
    private disconnect_adapter;
    private connect_socket_then_lsp;
    /**
     * Connect the change signal in order to update all virtual documents after a change.
     *
     * Update to the state of a notebook may be done without a notice on the CodeMirror level,
     * e.g. when a cell is deleted. Therefore a JupyterLab-specific signals are watched instead.
     *
     * While by not using the change event of CodeMirror editors we loose an easy way to send selective,
     * (range) updates this can be still implemented by comparison of before/after states of the
     * virtual documents, which is even more resilient and -obviously - editor-independent.
     */
    connect_contentChanged_signal(): void;
    handle_jump(locations: lsProtocol.Location[], id_path: string): void;
    create_adapter(virtual_document: VirtualDocument, connection: LSPConnection): CodeMirrorAdapterExtension;
    update_documents(slot: any): void;
    get_position_from_context_menu(): IRootPosition;
    get_context_from_context_menu(): IContext;
    protected create_tooltip(markup: lsProtocol.MarkupContent, cm_editor: CodeMirror.Editor, position: IEditorPosition): FreeTooltip;
}
export {};
