"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const jupyterlab_1 = require("./jupyterlab");
const notebook_1 = require("../virtual/editors/notebook");
const utils_1 = require("../utils");
const completion_1 = require("../completion");
const defaults_1 = require("../magics/defaults");
const defaults_2 = require("../extractors/defaults");
class NotebookAdapter extends jupyterlab_1.JupyterLabWidgetAdapter {
    constructor(editor_widget, jumper, app, completion_manager, rendermime_registry) {
        super(app, editor_widget, rendermime_registry, 'completer:invoke-notebook');
        this.editor = editor_widget.content;
        this.completion_manager = completion_manager;
        this.jumper = jumper;
        this.init_once_ready().then();
    }
    is_ready() {
        return (this.widget.context.isReady &&
            this.widget.content.isVisible &&
            this.widget.content.widgets.length > 0 &&
            this.language !== '');
    }
    get document_path() {
        return this.widget.context.path;
    }
    get language() {
        let language_metadata = this.widget.model.metadata.get('language_info');
        // @ts-ignore
        return language_metadata.name;
    }
    find_ce_editor(cm_editor) {
        return this.virtual_editor.cm_editor_to_cell.get(cm_editor).editor;
    }
    init_once_ready() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('LSP: waiting for', this.document_path, 'to fully load');
            yield utils_1.until_ready(this.is_ready.bind(this), -1);
            console.log('LSP:', this.document_path, 'ready for connection');
            // TODO
            // this.widget.context.pathChanged
            this.virtual_editor = new notebook_1.VirtualEditorForNotebook(this.widget, this.language, defaults_1.language_specific_overrides, defaults_2.foreign_code_extractors, this.document_path);
            this.connect_contentChanged_signal();
            // register completion connectors on cells
            this.document_connected.connect(() => this.connect_completion());
            yield this.connect_document(this.virtual_editor.virtual_document);
        });
    }
    connect_completion() {
        // see https://github.com/jupyterlab/jupyterlab/blob/c0e9eb94668832d1208ad3b00a9791ef181eca4c/packages/completer-extension/src/index.ts#L198-L213
        const cell = this.widget.content.activeCell;
        this.current_completion_connector = new completion_1.LSPConnector({
            editor: cell.editor,
            connections: this.connections,
            virtual_editor: this.virtual_editor,
            session: this.widget.session
        });
        const handler = this.completion_manager.register({
            connector: this.current_completion_connector,
            editor: cell.editor,
            parent: this.widget
        });
        this.widget.content.activeCellChanged.connect((notebook, cell) => {
            // TODO should those be cached? or the old ones disposed?
            this.current_completion_connector = new completion_1.LSPConnector({
                editor: cell.editor,
                connections: this.connections,
                virtual_editor: this.virtual_editor,
                session: this.widget.session
            });
            handler.editor = cell.editor;
            handler.connector = this.current_completion_connector;
        });
    }
}
exports.NotebookAdapter = NotebookAdapter;
