"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const regexp_1 = require("./regexp");
// TODO: make the regex code extractors configurable
exports.foreign_code_extractors = {
    // general note: to match new lines use [^] instead of dot, unless the target is ES2018, then use /s
    python: [
        //
        // R magics (non-standalone: the R code will always be in the same, single R-namespace)
        //
        new regexp_1.RegExpForeignCodeExtractor({
            language: 'R',
            pattern: '^%%R( .*?)?\n([^]*)',
            extract_to_foreign: '$2',
            keep_in_host: true,
            is_standalone: false
        }),
        new regexp_1.RegExpForeignCodeExtractor({
            language: 'R',
            pattern: '(^|\n)%R (.*)\n?',
            extract_to_foreign: '$2',
            keep_in_host: true,
            is_standalone: false
        }),
        //
        // Standalone IPython magics
        // (script magics are standalone, i.e. consecutive code cells with the same magic create two different namespaces)
        //
        new regexp_1.RegExpForeignCodeExtractor({
            language: 'python',
            pattern: '^%%(python|python2|python3|pypy)( .*?)?\n([^]*)',
            extract_to_foreign: '$3',
            keep_in_host: false,
            is_standalone: true
        }),
        new regexp_1.RegExpForeignCodeExtractor({
            language: 'perl',
            pattern: '^%%(perl)( .*?)?\n([^]*)',
            extract_to_foreign: '$3',
            keep_in_host: false,
            is_standalone: true
        }),
        new regexp_1.RegExpForeignCodeExtractor({
            language: 'ruby',
            pattern: '^%%(ruby)( .*?)?\n([^]*)',
            extract_to_foreign: '$3',
            keep_in_host: false,
            is_standalone: true
        }),
        new regexp_1.RegExpForeignCodeExtractor({
            language: 'sh',
            pattern: '^%%(sh)( .*?)?\n([^]*)',
            extract_to_foreign: '$3',
            keep_in_host: false,
            is_standalone: true
        }),
        new regexp_1.RegExpForeignCodeExtractor({
            language: 'html',
            pattern: '^%%(html --isolated)( .*?)?\n([^]*)',
            extract_to_foreign: '$3',
            keep_in_host: false,
            is_standalone: true
        }),
        //
        // IPython magics producing continuous documents (non-standalone):
        //
        new regexp_1.RegExpForeignCodeExtractor({
            language: 'js',
            pattern: '^%%(js|javascript)( .*?)?\n([^]*)',
            extract_to_foreign: '$3',
            keep_in_host: false,
            is_standalone: false
        }),
        new regexp_1.RegExpForeignCodeExtractor({
            language: 'html',
            pattern: '^%%(html)( .*?)?\n([^]*)',
            extract_to_foreign: '$3',
            keep_in_host: false,
            is_standalone: false
        }),
        new regexp_1.RegExpForeignCodeExtractor({
            language: 'latex',
            pattern: '^%%(latex)( .*?)?\n([^]*)',
            extract_to_foreign: '$3',
            keep_in_host: false,
            is_standalone: false
        }),
        new regexp_1.RegExpForeignCodeExtractor({
            language: 'markdown',
            pattern: '^%%(markdown)( .*?)?\n([^]*)',
            extract_to_foreign: '$3',
            keep_in_host: false,
            is_standalone: false
        })
    ]
};
