"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// TODO: needs heavy unit testing
function position_at_offset(offset, lines) {
    let line = 0;
    let column = 0;
    for (let text_line of lines) {
        // each line has a new line symbol which is accounted for in offset!
        if (text_line.length + 1 <= offset) {
            offset -= text_line.length + 1;
            line += 1;
        }
        else {
            column = offset;
            break;
        }
    }
    return { line, column };
}
exports.position_at_offset = position_at_offset;
class RegExpForeignCodeExtractor {
    constructor(options) {
        this.language = options.language;
        this.options = options;
        this.global_expression = new RegExp(options.pattern, 'g');
        this.test_expression = new RegExp(options.pattern, 'g');
        this.expression = new RegExp(options.pattern);
        this.standalone = this.options.is_standalone;
    }
    has_foreign_code(code) {
        let result = this.test_expression.test(code);
        this.test_expression.lastIndex = 0;
        return result;
    }
    extract_foreign_code(code) {
        let lines = code.split('\n');
        let extracts = new Array();
        let started_from = this.global_expression.lastIndex;
        let match = this.global_expression.exec(code);
        let host_code_fragment;
        while (match !== null) {
            let matched_string = match[0];
            let foreign_code_fragment = matched_string.replace(this.expression, this.options.extract_to_foreign);
            // NOTE:
            // match.index + matched_string.length === this.sticky_expression.lastIndex
            let end = this.global_expression.lastIndex;
            if (this.options.keep_in_host) {
                host_code_fragment = code.substring(started_from, end);
            }
            else {
                if (started_from === match.index) {
                    host_code_fragment = '';
                }
                else {
                    host_code_fragment = code.substring(started_from, match.index) + '\n';
                }
            }
            // TODO: this could be slightly optimized (start at start) by using the match[n],
            //  where n is the group to be used; while this reduces the flexibility of extract_to_foreign,
            //  it might be better to enforce such strict requirement
            let start = match.index + matched_string.indexOf(foreign_code_fragment);
            extracts.push({
                host_code: host_code_fragment,
                foreign_code: foreign_code_fragment,
                range: {
                    start: position_at_offset(start, lines),
                    end: position_at_offset(start + foreign_code_fragment.length, lines)
                }
            });
            started_from = this.global_expression.lastIndex;
            match = this.global_expression.exec(code);
        }
        if (started_from !== code.length) {
            let final_host_code_fragment = code.substring(started_from, code.length);
            extracts.push({
                host_code: final_host_code_fragment,
                foreign_code: null,
                range: null
            });
        }
        return extracts;
    }
}
exports.RegExpForeignCodeExtractor = RegExpForeignCodeExtractor;
