/// <reference types="@types/codemirror" />
/// <reference types="@types/codemirror/codemirror-showhint" />
/// <reference types="codemirror/searchcursor" />
import { Notebook, NotebookPanel } from '@jupyterlab/notebook';
import { Cell } from '@jupyterlab/cells';
import { ShowHintOptions } from 'codemirror';
import { IOverridesRegistry } from '../../magics/overrides';
import { IForeignCodeExtractorsRegistry } from '../../extractors/types';
import { VirtualEditor } from '../editor';
import CodeMirror = require('codemirror');
import { IEditorPosition, IRootPosition, IVirtualPosition } from '../../positioning';
export declare class VirtualEditorForNotebook extends VirtualEditor {
    notebook: Notebook;
    notebook_panel: NotebookPanel;
    cell_to_corresponding_source_line: Map<Cell, number>;
    cm_editor_to_cell: Map<CodeMirror.Editor, Cell>;
    language: string;
    constructor(notebook_panel: NotebookPanel, language: string, overrides_registry: IOverridesRegistry, foreign_code_extractors: IForeignCodeExtractorsRegistry, path: string);
    transform_from_notebook_to_root(cell: Cell, position: CodeMirror.Position): IRootPosition;
    transform_editor_to_root(cm_editor: CodeMirror.Editor, position: IEditorPosition): IRootPosition;
    transform_from_root_to_editor(pos: IRootPosition): CodeMirror.Position;
    get_editor_index(position: IVirtualPosition): number;
    get_cm_editor(position: IRootPosition): CodeMirror.Editor;
    showHint: (options: ShowHintOptions) => void;
    state: any;
    addKeyMap(map: string | CodeMirror.KeyMap, bottom?: boolean): void;
    addLineClass(line: any, where: string, _class_: string): CodeMirror.LineHandle;
    addLineWidget(line: any, node: HTMLElement, options?: CodeMirror.LineWidgetOptions): CodeMirror.LineWidget;
    addOverlay(mode: any, options?: any): void;
    addPanel(node: HTMLElement, options?: CodeMirror.ShowPanelOptions): CodeMirror.Panel;
    charCoords(pos: IRootPosition, mode?: 'window' | 'page' | 'local'): {
        left: number;
        right: number;
        top: number;
        bottom: number;
    };
    coordsChar(object: {
        left: number;
        top: number;
    }, mode?: 'window' | 'page' | 'local'): IRootPosition;
    cursorCoords(where?: boolean, mode?: 'window' | 'page' | 'local'): {
        left: number;
        top: number;
        bottom: number;
    };
    cursorCoords(where?: IRootPosition | null, mode?: 'window' | 'page' | 'local'): {
        left: number;
        top: number;
        bottom: number;
    };
    readonly any_editor: CodeMirror.Editor;
    defaultCharWidth(): number;
    defaultTextHeight(): number;
    endOperation(): void;
    execCommand(name: string): void;
    getDoc(): CodeMirror.Doc;
    get_editor_at_root_line(pos: IRootPosition): CodeMirror.Editor;
    getTokenAt(pos: IRootPosition, precise?: boolean): CodeMirror.Token;
    getTokenTypeAt(pos: IRootPosition): string;
    get_cell_at(pos: IVirtualPosition): Cell;
    get_editor_at_virtual_line(pos: IVirtualPosition): CodeMirror.Editor;
    protected perform_documents_update(): void;
    getWrapperElement(): HTMLElement;
    heightAtLine(line: any, mode?: 'window' | 'page' | 'local', includeWidgets?: boolean): number;
    isReadOnly(): boolean;
    lineAtHeight(height: number, mode?: 'window' | 'page' | 'local'): number;
    off(eventName: string, handler: (doc_or_instance: any, ...args: any[]) => void, ...args: any[]): void;
    private _event_wrappers;
    on(eventName: string, handler: (doc_or_instance: any, ...args: any[]) => void, ...args: any[]): void;
    addEventListener(type: string, listener: EventListenerOrEventListenerObject): void;
    forEveryBlockEditor(callback: (cm_editor: CodeMirror.Editor) => any, monitor_for_new_blocks?: boolean): void;
}
